/*
 *  My Certificate Wizard
 *
 *  Copyright (C) 2004 Vlada Macek <mycert@seznam.cz>
 *
 ** $Id: misc.c,v 1.4 2004/10/19 12:57:54 tuttle Exp $ **
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program (see the file COPYING included with this
 *  distribution); if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <shlobj.h>
#include <string.h>
#include <unistd.h>
#include <windows.h>

#include "mycert-res.h"
#include "mycert.h"

extern GLOBALS globals;

/* Two request.c "callbacks". */
char *CfgGetValue(char *key)
{
	static char value[MAXLINELEN+1];

	GetPrivateProfileString("openssl", key, NULL, value, sizeof(value), globals.MyConfig);
	value[sizeof(value)-1] = '\0';

#ifdef DEBUG
	printf("CfgGetValue: %s -> %s\n", key, value ? value : "(null)");
#endif
	return value;
}

char *DlgGetValue(int key)
{
	static char value[MAXLINELEN+1];

	GetDlgItemText(globals.myH, key, value, sizeof(value));
	value[sizeof(value)-1] = '\0';

#ifdef DEBUG
	printf("DlgGetValue: %d -> %s\n", key, value ? value : "(null)");
#endif
	return value;
}

/* make the string safe to be passed somewhere */
void safe_string(char *cp)
{
	int c;
	while ((c = *cp)) {
		if (((c >= 'a') && (c <= 'z')) || ((c >= 'A') && (c <= 'Z')) ||
			((c >= '0') && (c <= '9')) || c == '-') ;
		else *cp = '_';

		++cp;
	}
}

/* show the Last Error in message box then exit the application with an error */
void ErrorExit(char *lpszFunction) 
{ 
	LPVOID lpMsgBuf;
	DWORD dw = GetLastError(); 

	FormatMessage(
		FORMAT_MESSAGE_ALLOCATE_BUFFER | 
		FORMAT_MESSAGE_FROM_SYSTEM,
		NULL,
		dw,
		MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
		(LPTSTR) &lpMsgBuf,
		0, NULL);

	MsgBox("Fatal Error", MB_OK | MB_ICONERROR, "%s failed with error %d: %s",
		lpszFunction, (int)dw, (char *)lpMsgBuf); 

	LocalFree(lpMsgBuf);
	ExitProcess(dw); 
}

void DestroyBillGatesEmpire()
{
	/* To be implemented. */
}

int CALLBACK _ChooseFolderFunc(HWND hWnd, UINT uMsg, LPARAM lParam, LPARAM lpData)
{
	if (uMsg==BFFM_INITIALIZED)
		SendMessage(hWnd, BFFM_SETSELECTION, TRUE, lpData);

	return 0;
}

BOOL ChooseFolder(HWND owner, char *pszDir)
{
	BROWSEINFO bi;
	LPITEMIDLIST pidlBrowse;

	bi.hwndOwner = owner;
	bi.pidlRoot = NULL;
	bi.lpszTitle = "Please select the folder where your new private key and request shall be saved:";
	bi.pszDisplayName = pszDir;
	bi.ulFlags = BIF_RETURNONLYFSDIRS;
	bi.lpfn = _ChooseFolderFunc;
	bi.lParam = (LPARAM)(LPCSTR)pszDir;

	pidlBrowse = SHBrowseForFolder(&bi);
	if (pidlBrowse && SHGetPathFromIDList(pidlBrowse, pszDir))
		return TRUE;
	return FALSE;
} 

BOOL CALLBACK DONEDialogFunc(HWND hwndDlg, UINT msg, WPARAM wParam, LPARAM lParam)
{
  HICON hIcon;
  RECT rect;
  FILE *fp;
  LOGFONT logFont;
  HFONT hFont;
  char *p;

  switch (msg) {

    case WM_INITDIALOG:
	hIcon = (HICON)LoadImage(GetModuleHandle(NULL),
		MAKEINTRESOURCE(APP_ICON), IMAGE_ICON, 0, 0, LR_DEFAULTCOLOR);
	if (hIcon) {
		SendMessage(hwndDlg, WM_SETICON, (WPARAM) (ICON_SMALL), (LPARAM) (hIcon));
		SendMessage(hwndDlg, WM_SETICON, (WPARAM) (ICON_BIG), (LPARAM) (hIcon));
	}

	GetWindowRect(hwndDlg, &rect);
	MoveWindow(hwndDlg,	rect.left,
				rect.top + 100,
				rect.right - rect.left,
				rect.bottom - rect.top, FALSE);

	SetDlgItemText(hwndDlg, TEXT_PATH, globals.msgbuf);

	GetObject(GetStockObject(DEFAULT_GUI_FONT), sizeof(logFont), &logFont);
 	logFont.lfWeight = FW_BOLD;
  	hFont = CreateFontIndirect(&logFont);
	SendDlgItemMessage(hwndDlg, TEXT_WARNING, WM_SETFONT, (WPARAM)hFont, 0);

	GetObject(GetStockObject(DEFAULT_GUI_FONT), sizeof(logFont), &logFont);
 	lstrcpy(logFont.lfFaceName, TEXT("Fixedsys"));
  	hFont = CreateFontIndirect(&logFont);
	/* I don't know where to destroy this font and whether it is neccessary. */

	sprintf(globals.msgbuf, "Filename: %s\r\n", globals.OutFilePath);
	p = globals.msgbuf + strlen(globals.msgbuf);

	fp = fopen(globals.OutFilePath, "r");
	if (fp) {
		char *e = globals.msgbuf + sizeof(globals.msgbuf);

		/* Read and convert LF -> CRLF */
		while(fgets(p, e - p, fp)) {
			if ((p = strchr(p, '\n'))) {
				*p++ = '\r';
				*p++ = '\n';
				if (p >= e) break;
			} else break;
		}
		globals.msgbuf[sizeof(globals.msgbuf)-1] = '\0';
		fclose(fp);

		SendDlgItemMessage(hwndDlg, TEXT_DONEMSG, WM_SETFONT, (WPARAM)hFont, 0);
		SetDlgItemText(hwndDlg, TEXT_DONEMSG, globals.msgbuf);

	} else SetDlgItemText(hwndDlg, TEXT_DONEMSG, "(Cannot read the file.)");

	return TRUE;

    case WM_COMMAND:
	switch (LOWORD(wParam)) {
	    case IDOK:
          	EndDialog(hwndDlg, LOWORD(wParam));
		return TRUE;
	    case IDCOPYBUT:
		SendDlgItemMessage(hwndDlg, TEXT_DONEMSG, EM_SETSEL, 0, -1);
		SendDlgItemMessage(hwndDlg, TEXT_DONEMSG, WM_COPY, 0, 0);
		return TRUE;
            case IDCANCEL:
		EndDialog(hwndDlg, LOWORD(wParam));
		return TRUE;
	}
	break;
    case WM_CLOSE:
	EndDialog(hwndDlg, LOWORD(wParam));
	return TRUE;   
  }
  return FALSE;
}

static void DisplayErrorToUser(int errorCode, const char *description, const char *fmt)
{
	char *lpMsgBuf = NULL, msgBuf[MAXLINELEN+1];

	FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_ALLOCATE_BUFFER,
		  NULL,
		  errorCode,
		  0,
		  (char*)&lpMsgBuf,
		  0, NULL);
    
	strncpy(msgBuf, lpMsgBuf, MAXLINELEN);
	MsgBox("Failure copying files", MB_OK, fmt, msgBuf);
	LocalFree(lpMsgBuf);
}

static BOOL CopyOneFile(char *source, char *target)
{
	if(!CopyFile(source, target, FALSE)) {
		char buffer[MAXLINELEN+1];
		int errorCode = GetLastError();

		snprintf(buffer, MAXLINELEN, "Unable to copy \"%s\" to \"%s\" - error %d: %%s", source, target, errorCode);
		DisplayErrorToUser(errorCode, "Failure copying files", buffer);
		return FALSE;
	}
	return TRUE;
}

static BOOL CopyFileToOutDir(char *filename)
{
	char target[MAX_PATH+1];

	char *basename = strrchr(filename, '\\');
	if (!basename) basename = filename;
	while (basename[0] == '\\') basename++;

	strncpy(target, globals.OutFolder, sizeof(target));
	strcat(target, "\\");
	strncat(target, basename, sizeof(target) - strlen(target));

	return CopyOneFile(filename, target);
}

BOOL CopyExtraFiles()
{
	char CopyFiles[10*MAX_PATH];
	char *pos, *end;
	BOOL success = TRUE;

	GetPrivateProfileString("paths", "copyfiles", "", CopyFiles, sizeof(CopyFiles), globals.MyConfig);
	if (!CopyFiles[0]) return FALSE;

	pos = CopyFiles;
	while(1) {
		end = strchr(pos, ',');
		if (!end) {
			if (pos[0] && !CopyFileToOutDir(pos))
				success = FALSE;
			break;
		}

		*end = 0;    
		if (!CopyFileToOutDir(pos)) success = FALSE;
		pos = end + 1;
	}
	return success;
}

static void escapeBackSlashes(int outBufLen, char *outBuffer, char *inString)
{
	char *inPos = inString,
	     *outPos = outBuffer,
	     *outEnd = outBuffer + outBufLen - 1;

	while(*inPos != 0 && outPos < outEnd) {
		if(*inPos != '\\') {
			*outPos++ = *inPos++;
		} else {
			*outPos++ = *inPos++;
			*outPos++ = '\\';
		}
	}
	*outPos++ = 0;
}

BOOL ProcessOVPNTemplate()
{
	BOOL success;
	char filename[MAX_PATH+1];
	
	success = CopyOneFile(globals.OVPNTemplate, globals.OVPNFilePath);

	if (success) {
		FILE *fp = fopen(globals.OVPNFilePath, "a");

		if (fp) {
			char keyFilePath[MAX_PATH+1], certFilePath[MAX_PATH+1];

			escapeBackSlashes(MAX_PATH, keyFilePath, globals.KeyFilePath);
			escapeBackSlashes(MAX_PATH, certFilePath, globals.CertFilePath);

			fprintf(fp, "\n\n### The following lines were added to the template\n### by the My Certificate Wizard.\nkey  \"%s\"\ncert \"%s\"\n### ---\n\n",
				keyFilePath, certFilePath);
			fclose(fp);
		} else
			success = FALSE;
	}

	return success;
}

void ResolveConfigFilename(char *forcepath)
{
	char *p;

	if (forcepath && forcepath[0])
		strcpy(globals.MyConfig, forcepath);
	else {
		if (!GetModuleFileName(NULL, globals.MyConfig, MAX_PATH))
			ErrorExit("GetModuleFileName");

		p = strrchr(globals.MyConfig, '.');
		if (p) *p = '\0';
		strcat(globals.MyConfig, ".ini");
	}

	if (access(globals.MyConfig, R_OK) != 0) {
		MsgBox("Warning", MB_OK | MB_ICONWARNING,
			"Expected INI file %s not found or is not readable.\nUsing default values.", globals.MyConfig);
	}

}
